﻿RepTracker = LibStub("AceAddon-3.0"):NewAddon("RepTracker", "AceConsole-3.0", "AceEvent-3.0")
local tracked
local RepTrackerVersion = "3.0.0"
local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)

local function tc(f, r)
    return f:upper()..r:lower()
end

local defaults = {
	faction = {
		showInChat = true,
		dynamic = true,
		sticky = true,
		decrease = false,
		reset = false,
        trackTabard = false,
        dungeonsOnly = false,
        guild = true,
        defaultGuild = true,
        minimap = {
            hide = false},
		ZoneToFaction = {}
	}
}

local options = {
	name = "RepTracker",
	handler = RepTracker,
	type = "group",
	args = {
        version = {
			order = 1,
			type = 'description',
			name = 'Version: |cFFFF8080'..RepTrackerVersion..'|r'
		},
		author = {
			order = 2,
			type = 'description',
			name = 'by |cFFFF8080InSo(v)nIaX|r'
		},
        trackheader = {
            order = 3,
            type = 'header',
            name = 'Tracking'
        },
		track = {
			order = 4,
			type = 'input',
			name = 'Current faction tracked:',
			desc = 'Sets which faction is tracked in this zone by default.  <none> to remove the default.',
			get = "GetZone",
			set = "changeBar",
            disabled = function() return RepTracker.db.faction.trackTabard end
		},
        trackTabard = {
            order = 5,
			type = 'toggle',
			name = 'Track Tabard',
			desc = 'Just track the reputation for the faction whose tabard you are wearing.',
			get = "Is",
			set = "Toggle"
        },
		trackDesc = {
			order = 6,
			type = 'description',
			name = 'Sets which faction is tracked in this zone by default.  <none> to remove the default.'
		},
		optionsHeader = {
			order = 7,
			type = 'header',
			name = 'Options'
		},
		dynamic = {
			order = 8,
			type = 'toggle',
			name = 'Dynamic Tracking',
			desc = 'The tracked reputation changes to reflect the last faction you gained reputation with.',
			get = "Is",
			set = "Toggle",
            disabled = function() return RepTracker.db.faction.trackTabard end
		},
		dynamicDesc = {
			order = 9,
			type = 'description',
			name = 'The tracked reputation changes to reflect the last faction you gained reputation with.'
		},
		sticky = {
			order = 10,
			type = 'toggle',
			name = 'Sticky Rep',
			desc = 'Dynamic tracking changes are permanently stored as the default tracked faction for that zone.',
			get = "Is",
			set = "Toggle",
			disabled = function()
                if not RepTracker.db.faction.dynamic or RepTracker.db.faction.trackTabard then
                    return true
                else
                    return false
                end
            end
		},
		stickyDesc = {
			order = 11,
			type = 'description',
			name = 'Dynamic tracking changes are permanently stored as the default tracked faction.',
            disabled = function() return RepTracker.db.faction.trackTabard end
		},
		showInChat = {
			order = 12,
			type = 'toggle',
			name = 'Show In Chat',
			desc = 'Display RepTracker messages in the chat window.',
			get = "Is",
			set = "Toggle"
		},
		showInChatDesc = {
			order = 13,
			type = 'description',
			name = 'Display RepTracker messages in the chat window.'
		},		
		decrease = {
			order = 14,
			type = 'toggle',
			name = 'Track Decreases',
			desc = 'Track reputation decreases.',
			get = "Is",
			set = "Toggle",
            disabled = function() return RepTracker.db.faction.trackTabard end
		},
		decreaseDesc = {
			order = 15,
			type = 'description',
			name = 'Track reputation decreases.'
		},
        guild = {
            order = 16,
            type = 'toggle',
            name = 'Track Guild Reputation',
            desc = 'Track Guild reputation changes',
            get = "Is",
            set = "Toggle",
            disabled = function() return RepTracker.db.faction.trackTabard end
        },
        guildDesc = {
            order = 17,
            type = 'description',
            name = 'Includes changes to guild reputation in tracking.'
        },
        defaultGuild = {
            order = 18,
            type = 'toggle',
            name = 'Default to Guild',
            desc = 'Tracks Guild reputation when entering a zone that does not yet have a faction assigned.',
            get = "Is",
            set = "Toggle",
            disabled = function()
                if RepTracker.db.faction.trackTabard or RepTracker.db.faction.reset then
                    return true
                else
                    return false
                end
            end
        },
        defaultGuildDesc = {
            order = 19,
            type = 'description',
            name = 'Tracks Guild reputation when entering a zone that does not yet have a faction assigned.'
        },
		reset = {
			order = 20,
			type = 'toggle',
			name = 'Faction Reset',
			desc = 'Clears the tracked faction upon entering a zone if that zone does not yet have a faction assigned.',
			get = "Is",
			set = "Toggle",
            disabled = function()
                if RepTracker.db.faction.trackTabard or RepTracker.db.faction.defaultGuild then
                    return true
                else
                    return false
                end
            end
		},
		resetDesc = {
			order = 21,
			type = 'description',
			name = 'Clears the tracked faction upon entering a zone if that zone does not yet have a faction assigned.'
		},
        startup = {
			order = 22,
			type = 'toggle',
            name = 'Show Version Loaded',
            desc = 'Show the one-line startup text that displays RepTracker version loaded.',
			get = "Is",
			set = "Toggle"
		},
		startupDesc = {
			order = 23,
			type = 'description',
			name = 'Show the one-line startup text that displays RepTracker version loaded.'
		},
        minimap = {
			order = 24,
			type = 'toggle',
            name = 'Show Minimap Button',
            desc = 'Show the Minimap Button.',
			get = "Is",
			set = "Toggle"
		},
		minimapDesc = {
			order = 25,
			type = 'description',
            name = 'Show the Minimap Button.'
		}
	}
}

function RepTracker:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RepTrackerDB", defaults)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RepTracker", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RepTracker", "RepTracker")
	self:RegisterChatCommand("rt", "ChatCommand")
	self:RegisterChatCommand("RepTracker", "ChatCommand")
    RepTrackerLauncher = LDB:NewDataObject("RepTracker", {
	    type = "launcher",
	    icon = "Interface\\Icons\\Achievement_reputation_01.png",
		OnClick = function(_, button)
            local _, _, bg = GetWorldStateUIInfo(1)
            if button == "RightButton" then
                InterfaceOptionsFrame_OpenToCategory(RepTracker.optionsFrame)
            end
        end,
        OnTooltipShow = function(tt)
            tt:AddLine("|cFFFF0000RepTracker "..RepTrackerVersion)
            tt:AddLine("|cFFFF8080Right Click|cFFFFFFFF to display the options panel")
        end})
    LDBIcon:Register("RepTracker", RepTrackerLauncher, self.db.faction.minimap)
end

function RepTracker:OnEnable()
    CombatTextSetActiveUnit("player")
    if self.db.faction.dynamic and not self.db.faction.trackTabard then
        self:RegisterEvent("COMBAT_TEXT_UPDATE")
    end
    if self.db.faction.trackTabard then
        self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
        self:TabardCheck()
    else
        self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
    end
    if self.db.faction.startup then
        self:Print(RepTrackerVersion.." Loaded.")
    end
end

function RepTracker:TabardCheck()
    tracked = strlower(GetItemInfo(GetInventoryItemLink("player", 19)))
    self:ScanFactions()
    for i=1, getn(self.db.faction.tabardFactions) do
        if self.db.faction.tabardFactions[i] then
            if strmatch(tracked, self.db.faction.tabardFactions[i]) then
                SetWatchedFactionIndex(i)
                if self.db.faction.showInChat then
	                self:Print("Tracking "..self.db.faction.tabardFactions[i]:gsub("(%a)([%w_']*)", tc).." reputation")
			    end
            elseif strmatch(tracked, "guild") then
                SetWatchedFactionIndex(2)
                if self.db.faction.showInChat then
			        self:Print("Tracking "..select(1, GetGuildInfo("player")).." reputation")
			    end
                return
            end
        end
    end
end

function RepTracker:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    elseif strmatch(input, "track") then
        RepTracker:changeBar(_, input:gsub("track ", ""))
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(RepTracker, "rt", "RepTracker", input)
    end
end

function RepTracker:Is(info)
    if info[1] == "minimap" then
        return not self.db.faction.minimap.hide
    else
	    return self.db.faction[info[1]]
    end
end

function RepTracker:Toggle(info, value)
    if info[1] == "minimap" then
        self.db.faction.minimap.hide = not value
        if self.db.faction.minimap.hide then
            LDBIcon:Hide("RepTracker")
        else
            LDBIcon:Show("RepTracker")
        end
        return
    else
        self.db.faction[info[1]] = value
    end
    if info[1] == "dynamic" and self.db.faction.dynamic then
        self:RegisterEvent("COMBAT_TEXT_UPDATE")
    elseif info[1] == "dynamic" then
        self:UnregisterEvent("COMBAT_TEXT_UPDATE")
    elseif info[1] == "trackTabard" and self.db.faction.trackTabard then
        self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
        self:UnregisterEvent("COMBAT_TEXT_UPDATE")
        self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
        self:TabardCheck()
    elseif info[1] == "trackTabard" then
        self:ZONE_CHANGED_NEW_AREA()
        self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
        self:UnregisterEvent("PLAYER_EQUIPMENT_CHANGED")
        if self.db.faction.dynamic then
            self:RegisterEvent("COMBAT_TEXT_UPDATE")
        end
    end
end

function RepTracker:PLAYER_EQUIPMENT_CHANGED(_, slot, item)
    if slot == 19 and item == 1 then
        self:TabardCheck()
    end
end

function RepTracker:ScanFactions()
	local factions = {}
    local tabardFactions = {}
	local name, isHeader, isCollapsed
	factions["none"] = 0
	for i = 0, GetNumFactions() do
		_, _, _, _, _, _, _, _, isHeader, isCollapsed = GetFactionInfo(i)
		if isHeader and isCollapsed then
			ExpandFactionHeader(i)
		end
	end
	for i = 0, GetNumFactions() do
		name, _, _, _, _, _, _, _, isHeader = GetFactionInfo(i)
		if not isHeader and name ~= nil then
			factions[string.lower(name)] = i
            tabardFactions[i] = string.lower(name)
		end
	end
	self.db.faction.factions = factions
    self.db.faction.tabardFactions = tabardFactions
end

function RepTracker:ZONE_CHANGED_NEW_AREA()
	self:ScanFactions()
	local faction = self:GetZone()
	if faction == nil then
		if self.db.faction.showInChat then
            if not self.db.faction.defaultGuild then
			    self:Print(GetZoneText()..": no faction tracked")
			    self:Print("To set the tracked faction for this zone, type: /rt track <faction>")
            else
                self:Print(GetZoneText()..": no faction tracked; defaulting to Guild Reputation")
			    self:Print("To set the tracked faction for this zone, type: /rt track <faction>")
                self:ScanFactions()
                SetWatchedFactionIndex(2)
            end
		end
		if self.db.faction.reset then
			SetWatchedFactionIndex(0)
		end
	else
		faction = string.lower(faction)
		if self.db.faction.factions[faction] ~= nil then
			SetWatchedFactionIndex(self.db.faction.factions[faction])
			if self.db.faction.showInChat then
				self:Print(GetZoneText()..": tracking "..faction:gsub("(%a)([%w_']*)", tc).." reputation")
			end
		else
            if self.db.faction.showInChat then
			    self:Print("You have not made contact with "..faction:gsub("(%a)([%w_']*)", tc).." yet; tracking none")
            end
			if self.db.faction.reset then
				SetWatchedFactionIndex(0)
			end
		end
	end
end

function RepTracker:GetZone()
	return self.db.faction.ZoneToFaction[GetZoneText()]
end

function RepTracker:COMBAT_TEXT_UPDATE(_, messageType, faction, amount)
    if messageType ~= "FACTION" then
		return
	elseif amount < 0 and self.db.faction.dynamic and self.db.faction.decrease then
		self:changeBar(_, faction)
	elseif amount > 0 and self.db.faction.dynamic then
		self:changeBar(_, faction)
	end
end

function RepTracker:changeBar(_, faction)
    if faction == nil or faction == "none" then
        if self.db.faction.showInChat and self.db.faction.ZoneToFaction[GetZoneText()] ~= nil then
			self:Print("Not tracking a reputation")
        end
        SetWatchedFactionIndex(0)
        self.db.faction.ZoneToFaction[GetZoneText()] = nil
        return
    end
    self:ScanFactions()
    if GetWatchedFactionInfo() then
        oldName = strlower(GetWatchedFactionInfo())
    end
    if strlower(faction) == "guild" and self.db.faction.guild then
        if select(1,GetWatchedFactionInfo()) ~= select(1,GetGuildInfo("player")) then
            SetWatchedFactionIndex(2)
            if self.db.faction.showInChat then
                if self.db.faction.showInChat then
	                self:Print("Now tracking "..select(1,GetGuildInfo("player")).." reputation")
                end
            end
        end
        return
	end
	for i=1, GetNumFactions() do
		local name = strlower(GetFactionInfo(i))
		if name == strlower(faction) and name ~= oldName and name ~= "alliance" and name ~= "horde" and name ~= "guild" then
			SetWatchedFactionIndex(i)
			if self.db.faction.showInChat then
                if self.db.faction.trackTabard then
				    self:Print("Now tracking "..name:gsub("(%a)([%w_']*)", tc).." reputation")
                else
                    self:Print(GetZoneText()..": now tracking "..name:gsub("(%a)([%w_']*)", tc).." reputation")
                end
			end
			if self.db.faction.sticky then
				self.db.faction.ZoneToFaction[GetZoneText()] = string.lower(faction)
	        end
            return
        end
	end
end